#include "InterprocessorCommunication\InterprocessorCommunication_ByteQueue.c" 
// some definitions.  Please see the report for the table of these signals and 
//  how they relate to the protocol.  It is too extensive to list here ... 
#define DInLeft PIND.4
#define ReadyOutLeft PORTD.6
#define DInRight PINA.3
#define ReadyOutRight PORTA.5
#define ReadBusInRight PINA.6
#define ReadBusInLeft PIND.7
#define IsConnectedToRight PINA.7
#define IsConnectedToLeft PINA.0
#define DOutRight PORTA.4
#define DOutLeft PORTD.5
#define ReadyInLeft PIND.2
#define ReadyInRight PIND.3


// Variables necessary for byte communication
char ReceiveByteBufferFromLeft;		// the current byte that we're receiving (from the left)
char ReceiveByteBufferFromRight;	// the current byte that we're receiving (from the right)
char ReceiveByteBufferFromLeft_CurrentStep;		// current bit index of the byte (see above)
char ReceiveByteBufferFromRight_CurrentStep;	// current bit index of the byte (see above)
char IsConnectedToLeftS;	// static definition (that only changes every 30ms) of whether we're connected
char IsConnectedToRightS;	// since we poll, we need a static variable (as in, not just the dynamic pin.
							// This is meant to reduce bouncing, but i can't put the wires in fast enough
void InterProcessorCommunications_OnConnectedToLeftChanged();
void InterProcessorCommunications_OnConnectedToRightChanged();
/*
	Event handler that handles an incoming byte from the left.
	See the report for protocol description.

	Right after it is an event handler that handles an incoming byte from the left.
	It is nearly identical.
*/
interrupt [EXT_INT0] void receive_pack_inter_from_left(void)
{
	ReceiveByteBufferFromLeft |= ReadBusInLeft << ReceiveByteBufferFromLeft_CurrentStep;

	if(ReceiveByteBufferFromLeft_CurrentStep == 7)
	{                         
		// we received a byte, put it in a mailbox ... 
		//  if there is no available mailbox, we're going to freeze ... not good!
		//  note, interupts are currently disabled within an interupt handler!
		InterProcessorCommunicationsQueue_AddData(ReceiveByteBufferFromLeft, 0);
		ReceiveByteBufferFromLeft_CurrentStep = 0;
		ReceiveByteBufferFromLeft = 0;
	}
	else
	{
		ReceiveByteBufferFromLeft_CurrentStep += 1;
	}
	// the handshake
	DOutLeft = 1;
	while(ReadyInLeft != 0)
	if(!IsConnectedToLeft) // fail-fast
	{  
		// reset everything and break the loop
		ReceiveByteBufferFromLeft_CurrentStep = 0;
		ReceiveByteBufferFromLeft = 0;
		break;
	}
	DOutLeft = 0;
}
interrupt [EXT_INT1] void receive_pack_inter_from_right(void)
{
	ReceiveByteBufferFromRight |= ReadBusInRight << ReceiveByteBufferFromRight_CurrentStep;

	if(ReceiveByteBufferFromRight_CurrentStep == 7)
	{                         
		// we received a byte, put it in a mailbox ... 
		//  if there is no available mailbox, we're going to freeze ... not good!
		//  note, interupts are currently disabled within an interupt handler!
		InterProcessorCommunicationsQueue_AddData(ReceiveByteBufferFromRight, 1);
		ReceiveByteBufferFromRight_CurrentStep = 0;
		ReceiveByteBufferFromRight = 0;
	}
	else
	{
		ReceiveByteBufferFromRight_CurrentStep += 1;
	}
	DOutRight = 1;
	while(ReadyInRight != 0)
	if(!IsConnectedToRight)
	{  
		// reset everything and break the loop
		ReceiveByteBufferFromRight_CurrentStep = 0;
		ReceiveByteBufferFromRight = 0;
		break;
	}
	DOutRight = 0;
}
/*
	Before calling any of these function, make sure that there are enough
	 bytes in the buffer to actually be read.  Otherwise, this might leave the queues, in 
	 and inconsistant state.
	There are functions here both for reading a single byte and for reading a sequence of bytes.
*/
inline char InterProcessorCommunication_ReceiveByte(char d)	{return InterProcessorCommunicationsQueue_ReadData(d);}
inline char InterProcessorCommunication_ReceiveByte_Left()	{return InterProcessorCommunication_ReceiveByte(0);}
inline char InterProcessorCommunication_ReceiveByte_Right()	{return InterProcessorCommunication_ReceiveByte(1);}                                      
void InterProcessorCommunication_ReceiveBytes(char size, char * bytes, char dir)	{int i; for(i=0;i<size;i++) bytes[i] = InterProcessorCommunication_ReceiveByte(dir);}
inline void InterProcessorCommunication_ReceiveBytes_Left(char size, char * bytes)	{InterProcessorCommunication_ReceiveBytes(size, bytes, 0);}
inline void InterProcessorCommunication_ReceiveBytes_Right(char size, char * bytes)	{InterProcessorCommunication_ReceiveBytes(size, bytes, 1);}
/*
	Byte level handshaking from the sending side.  See the report for more details on the protocol.
*/
inline void HandShake_Sender_Left() {
	ReadyOutLeft = 1;
	while(DInLeft != 1 && IsConnectedToLeft) ; // wait for DInLeft to be 1
	ReadyOutLeft = 0;
	while(DInLeft != 0 && IsConnectedToLeft) ; // wait for DInLeft to be 0
}
inline void HandShake_Sender_Right() {
	ReadyOutRight = 1;
	while(DInRight == 0 && IsConnectedToRight) ;
	ReadyOutRight = 0;
	while(DInRight == 1 && IsConnectedToRight) ;
}
// sends a byte (either left [driection=0] of right [direction=1]) syncronously!
char SendByte(char b, char direction)
{    
	char i;
	if(direction == 0) // send it left
	{ 
		for(i=0;i<8;i++)
		{
			PORTD.0 = (b) & (0x01 << i);
			HandShake_Sender_Left();
			if(!IsConnectedToLeft) return 0;
		}
	}
	else // direction = 1
	{
		PORTA = (PORTA & 0xFD) | ( 0x02 & (b<<1) );		HandShake_Sender_Right(); if(!IsConnectedToRight) return 0;
		PORTA = (PORTA & 0xFD) | ( 0x02 & (b>>0) );		HandShake_Sender_Right(); if(!IsConnectedToRight) return 0;	
		PORTA = (PORTA & 0xFD) | ( 0x02 & (b>>1) );		HandShake_Sender_Right(); if(!IsConnectedToRight) return 0;	
		PORTA = (PORTA & 0xFD) | ( 0x02 & (b>>2) );		HandShake_Sender_Right(); if(!IsConnectedToRight) return 0;
		PORTA = (PORTA & 0xFD) | ( 0x02 & (b>>3) );		HandShake_Sender_Right(); if(!IsConnectedToRight) return 0;	
		PORTA = (PORTA & 0xFD) | ( 0x02 & (b>>4) );		HandShake_Sender_Right(); if(!IsConnectedToRight) return 0;	
		PORTA = (PORTA & 0xFD) | ( 0x02 & (b>>5) );		HandShake_Sender_Right(); if(!IsConnectedToRight) return 0;
		PORTA = (PORTA & 0xFD) | ( 0x02 & (b>>6) );		HandShake_Sender_Right(); if(!IsConnectedToRight) return 0;
	}     
	return 1;
}
// sends a set of bytes (direction is the same as above).
char InterProcessorCommunication_SendBytes(char * bytes, char numberOfBytes, char direction)
{ 
	int i;
	for(i=0;i<numberOfBytes;i++)
	{                                            
		if(!SendByte(bytes[i], direction)) return 0;
	}
	return 1;
}
/*
	Check for connections and disconnections and update the static variables that
	 keep track of the current state.
*/
void InterprocessorCommunication_TimerInteruptHandler( void )
{
	if(IsConnectedToLeftS != IsConnectedToLeft)
	{
		IsConnectedToLeftS = IsConnectedToLeft;
		InterProcessorCommunications_OnConnectedToLeftChanged();
	}
	if(IsConnectedToRightS != IsConnectedToRight)
	{
		IsConnectedToRightS = IsConnectedToRight;
		InterProcessorCommunications_OnConnectedToRightChanged();
	}
}
// Initialization method, should be called from startup initialization method.
void InterProcessorCommunication_ByteCommunication_Initialize()
{
  IsConnectedToLeftS = 0;
  IsConnectedToRightS = 0;                        
  ReceiveByteBufferFromLeft_CurrentStep = 0;
  ReceiveByteBufferFromRight_CurrentStep = 0;
  InterProcessorCommunicationsQueue_Initialize(); 
}