/*
	In order to keep the queue in a consistant state
	and since it needs to be accessed from both interupts a normal code
	we disable interupts when we have to modify the queue.  Therefore
	all methods are "transactional" (actually you only read and write transactionally with
	respect to each other, therefore only read and write are protected
	the has data function is not, because all reads are syncronous with all 
	other reads.

	these critical section definitions are taken from the operating system
	(perhaps the only actual code that we borrowed)
*/
#define AOS_ENTER_CRITICAL  {    \
								#asm("cli")     \
        						aos_crit_count++; \
              				}
              				
#define AOS_EXIT_CRITICAL   {    \
								--aos_crit_count;  \
        						if(!aos_crit_count){	\
              						#asm("sei")    \
                    			}               	\
                       		}
int aos_crit_count;

#define NULL (void*)0 
#define Data_Size 10 // spots in the queue

// we need two of them, one for bytes from the right and one for bytes from the left.
//  this allows our framework to have maximum utility.
volatile char InterProcessorCommunicationsQueue_Data_FromLeft[Data_Size];
volatile char InterProcessorCommunicationsQueue_Data_FromRight[Data_Size];
volatile char InterProcessorCommunicationsQueue_FirstIndex_FromLeft;
volatile char InterProcessorCommunicationsQueue_FirstIndex_FromRight;
volatile char InterProcessorCommunicationsQueue_LastIndex_FromLeft;
volatile char InterProcessorCommunicationsQueue_LastIndex_FromRight;
volatile char InterProcessorCommunicationsQueue_Count_FromLeft;
volatile char InterProcessorCommunicationsQueue_Count_FromRight;
// initialization
void InterProcessorCommunicationsQueue_Initialize(void)
{
	InterProcessorCommunicationsQueue_FirstIndex_FromLeft	= 0;
	InterProcessorCommunicationsQueue_FirstIndex_FromRight	= 0;
	InterProcessorCommunicationsQueue_LastIndex_FromLeft	= 0xFF;
	InterProcessorCommunicationsQueue_LastIndex_FromRight	= 0xFF;
	InterProcessorCommunicationsQueue_Count_FromLeft		= 0;
	InterProcessorCommunicationsQueue_Count_FromRight		= 0;
}
// dir == 1 is from right side, dir == 0 is from left side.
// this method should be private, but this is "C"
void InterProcessorCommunicationsQueue_AddData2(
	char d, char* data, char* firstIndex, char* lastIndex, char* sem)
{
	char Temp;
	AOS_ENTER_CRITICAL;
	Temp = ((*lastIndex)+1) & 0xFF;
	if(Temp == Data_Size) Temp = 0;

	if(Temp == (*firstIndex) && (*lastIndex) != 0xFF) { // ?? wrong condition? 
		// we're full, assume that this never happens, dude
	}
	data[Temp] = d;
	*lastIndex = Temp;
	*sem = (*sem) + 1;
	AOS_EXIT_CRITICAL;
}
// calls the above method with the approriate pointers depending on the direction
// while this makes the code slower (pointes are not as fast to access)
// this does save on code space and allows for better readability.
void InterProcessorCommunicationsQueue_AddData(char d, char dir)
{    
	if(!dir)
		InterProcessorCommunicationsQueue_AddData2(d, 
			InterProcessorCommunicationsQueue_Data_FromLeft,
			&InterProcessorCommunicationsQueue_FirstIndex_FromLeft,
			&InterProcessorCommunicationsQueue_LastIndex_FromLeft,
			&InterProcessorCommunicationsQueue_Count_FromLeft);
	else
		InterProcessorCommunicationsQueue_AddData2(d, 
			InterProcessorCommunicationsQueue_Data_FromRight,
			&InterProcessorCommunicationsQueue_FirstIndex_FromRight,
			&InterProcessorCommunicationsQueue_LastIndex_FromRight,
			&InterProcessorCommunicationsQueue_Count_FromRight);
}
// check there before reading to check if there is data in the queue to read
//  otherwise the queue can be left in an inconstant state.
char InterProcessorCommunicationsQueue_HasData(char dir)
{
  return dir ? 
    (InterProcessorCommunicationsQueue_Count_FromRight > 0) : 
    InterProcessorCommunicationsQueue_Count_FromLeft > 0;
}
char InterProcessorCommunication_HasReadBytes(char i, char dir)
{
  i--;
  return dir ? 
    (InterProcessorCommunicationsQueue_Count_FromRight > i) : 
    InterProcessorCommunicationsQueue_Count_FromLeft > i;
}
char InterProcessorCommunication_HasReadByte(char dir) {return InterProcessorCommunication_HasReadBytes(1, dir);}

// alows you to read from the queue. use the non-2 method
char InterProcessorCommunicationsQueue_ReadData2(
	char* data, char* firstIndex, char* lastIndex, char* sem)
{
	char d;

	AOS_ENTER_CRITICAL;
	*sem = (*sem) - 1;
	d = data[*firstIndex];
	*firstIndex = (*firstIndex)+1;
	if(*firstIndex == Data_Size)
		*firstIndex = 0;
	AOS_EXIT_CRITICAL;
	return d;
}
// REMEMBER: must call "Has Data" first ... 
// calls the above method with the approriate pointers depending on the direction
// while this makes the code slower (pointes are not as fast to access)
// this does save on code space and allows for better readability.
char InterProcessorCommunicationsQueue_ReadData(char dir)  
{
	if(!dir)
		return InterProcessorCommunicationsQueue_ReadData2(
			InterProcessorCommunicationsQueue_Data_FromLeft,
			&InterProcessorCommunicationsQueue_FirstIndex_FromLeft,
			&InterProcessorCommunicationsQueue_LastIndex_FromLeft,
			&InterProcessorCommunicationsQueue_Count_FromLeft);
	else
		return InterProcessorCommunicationsQueue_ReadData2(
			InterProcessorCommunicationsQueue_Data_FromRight,
			&InterProcessorCommunicationsQueue_FirstIndex_FromRight,
			&InterProcessorCommunicationsQueue_LastIndex_FromRight,
			&InterProcessorCommunicationsQueue_Count_FromRight);
}