#include "InterprocessorCommunication\InterprocessorCommunication_ByteCommunication.c"
#define PacketDataSize 2
#define PacketHeaderSize 1
#define PacketSize (PacketDataSize + PacketHeaderSize)
#define ExtractPacketType(x) ( (x[0] >> 0) & 0x03 )
#define ExtractPacketFrom(x) ( (x[0] >> 2) & 0x07 )
#define ExtractPacketTo(x)   ( (x[0] >> 5) & 0x07 )
#define MakePacketHeader(type, from, to) ( type | (from << 2) | (to << 5) )



// Queue structure for incoming packets (not bytes)
// code for these methods is very similar to the ones for bytes,
//  see comments over there.
volatile unsigned int InterProcessorCommunicationsPacketQueue_Data[Data_Size];
volatile char InterProcessorCommunicationsPacketQueue_FirstIndex;
volatile char InterProcessorCommunicationsPacketQueue_LastIndex;
volatile char InterProcessorCommunicationsPacketQueue_Count;
// "IP" Address
char InterProcessorCommunications_ProcessorIP;                                            
void InterProcessorCommunication_SendControlPacket_IPReassigned() // can only ever send "right"
{                                                               
	char PacketBytes[PacketSize];
	// packet type is 0x03, for a clear-ip control packet
	PacketBytes[0] = (0 | 0x03) | (InterProcessorCommunications_ProcessorIP << 2); 
	// other bytes don't matter                                       
	// the below should fail if we're not connected.
	InterProcessorCommunication_SendBytes(PacketBytes, PacketSize, 1) ? 2 : 3;
} 

// methods very similar to the queue methods for bytes, see there.
void InterProcessorCommunicationsPacketQueue_AddData(unsigned int d)
{
	char Temp;
	Temp = (InterProcessorCommunicationsPacketQueue_LastIndex+1) & 0xFF;
	if(Temp == Data_Size) Temp = 0;

	if(Temp == (InterProcessorCommunicationsPacketQueue_FirstIndex) && 
		InterProcessorCommunicationsPacketQueue_LastIndex != 0xFF) { // ?? wrong condition? 
			// we're full, assume that this never happens, dude
	}
	InterProcessorCommunicationsPacketQueue_Data[Temp] = d;
	InterProcessorCommunicationsPacketQueue_LastIndex = Temp;
	InterProcessorCommunicationsPacketQueue_Count++;
}
char InterProcessorCommunicationPacketQueue_HasRead()
{
	return InterProcessorCommunicationsPacketQueue_Count > 0;
}
void InterProcessorCommunicationPacketQueue_ReadData(char * data)
{
	unsigned int d;
	InterProcessorCommunicationsPacketQueue_Count--;
	d = InterProcessorCommunicationsPacketQueue_Data[InterProcessorCommunicationsPacketQueue_FirstIndex];
	InterProcessorCommunicationsPacketQueue_FirstIndex++;
	if(InterProcessorCommunicationsPacketQueue_FirstIndex == Data_Size)
		InterProcessorCommunicationsPacketQueue_FirstIndex = 0;
	data[0] = d & 0xFF;
	data[1] = (d >> 8) & 0xFF;
}
// sends a packet.  The data should be 2 bytes long.
// to should be a number 1 to 7.  
char InterProcessorCommunication_SendPacket(char * data, char to)
{
	char PacketBytes[PacketSize];
	PacketBytes[0] = 0 | (to << 5) | (InterProcessorCommunications_ProcessorIP << 2);
	PacketBytes[1] = data[0];
	PacketBytes[2] = data[1];
	return InterProcessorCommunication_SendBytes(PacketBytes, PacketSize, to > InterProcessorCommunications_ProcessorIP);
}
/*
Handle dynamic connections, disconnections and reconnections
from either the left or the right
*/
void InterProcessorCommunications_OnConnectedToLeftChanged()
{
	// we were connected to the left, now we're not, so we're the left most
	//  therefore, we get number 1, and reassign processors on our right.
	if(!IsConnectedToLeftS)
	{
		InterProcessorCommunications_ProcessorIP = 1;
		InterProcessorCommunication_SendControlPacket_IPReassigned();
		return;
	}
	else
	{
		// our number will be "reassigned automatically" soon
	}
}
void InterProcessorCommunications_OnConnectedToRightChanged()
{
	if(IsConnectedToRightS) // meaning, we were just connected ... 
	{                                                              
		// we need the other processor (on the right) to be re-assigned ... 
		InterProcessorCommunication_SendControlPacket_IPReassigned();
		return;
	}
}
void InterProcessorCommunications_IPLayerThread_FromLeft()
{
	char PacketBytes[PacketSize];
	char PacketType;
	char PacketFrom;
	char PacketTo;
	unsigned int PacketData;
	// check to make sure there's a packet ... poll (in OS this would be done with semaphores.
	if(!InterProcessorCommunication_HasReadBytes(PacketSize, 0)) return;
	InterProcessorCommunication_ReceiveBytes_Left(PacketSize, PacketBytes);
	PacketType = ExtractPacketType(PacketBytes);
	PacketFrom = ExtractPacketFrom(PacketBytes);
	PacketTo   = ExtractPacketTo  (PacketBytes);
	if ( PacketTo == 0 )
	{
		// it's to us by default
		if(PacketType == 0x03) // it's a control packet and it means we have to reset our IP
		{ 
			InterProcessorCommunications_ProcessorIP = PacketFrom+1;
			InterProcessorCommunication_SendControlPacket_IPReassigned(); // tell guy on right to reassign
		}
	}
	else if ( PacketTo == InterProcessorCommunications_ProcessorIP ) // it was sent to us ... 
	{
		// add it to some list ...  
		PacketData = (((int)PacketBytes[2]) << 8); 
		PacketData |= ((int)PacketBytes[1]);
		InterProcessorCommunicationsPacketQueue_AddData(PacketData);
	}
	else
	{
		// send it out depending on where it's going ...       
		// if it's going to a processor that has a greater ID than us, it must be to our
		//  right, otherwise, it's to our left!
		InterProcessorCommunication_SendBytes(PacketBytes, PacketSize, 
			PacketTo > InterProcessorCommunications_ProcessorIP);
	}
}
// we never get an IP address reassign packet from the right (they only go to the right
// ... ie: from the left). so leave that code out of here.
void InterProcessorCommunications_IPLayerThread_FromRight()
{
	char PacketBytes[PacketSize];
	char PacketType;
	char PacketFrom;
	char PacketTo;
	unsigned int PacketData;                                             
	if(!InterProcessorCommunication_HasReadBytes(PacketSize, 1)) return;
	InterProcessorCommunication_ReceiveBytes_Right(PacketSize, PacketBytes);
	PacketType = ExtractPacketType(PacketBytes);
	PacketFrom = ExtractPacketFrom(PacketBytes);
	PacketTo   = ExtractPacketTo  (PacketBytes);
	if ( PacketTo == 0 ) { } // Currently doesn't happen ...
	else if ( PacketTo == InterProcessorCommunications_ProcessorIP ) // it was sent to us ... 
	{
		// add it to some list ...
		PacketData = (((int)PacketBytes[2]) << 8); 
		PacketData |= ((int)PacketBytes[1]);
		InterProcessorCommunicationsPacketQueue_AddData(PacketData);		 
	}
	else
	{
		// send it out depending on where it's going ...       
		// if it's going to a processor that has a greater ID than us, it must be to our
		//  right, otherwise, it's to our left!                
		//  note: since we got it from the right, it should be going left, but still test
		//  just to make sure ... 
		InterProcessorCommunication_SendBytes(PacketBytes, PacketSize, 
			PacketTo > InterProcessorCommunications_ProcessorIP);
	}
}
// initialization code.
void InterProcessorCommunication_IPCommunication_Initialize()
{
	InterProcessorCommunication_ByteCommunication_Initialize();
	InterProcessorCommunications_ProcessorIP = 1;               
	InterProcessorCommunicationsPacketQueue_FirstIndex = 0;
	InterProcessorCommunicationsPacketQueue_LastIndex = 0xFF;
	InterProcessorCommunicationsPacketQueue_Count = 0;
}