
//used as an example in the program organization doc.
#include <stdlib.h>                  
#include <stdio.h>
#include <Mega32.h> 
#include <delay.h>
#include "InterprocessorCommunication\InterprocessorCommunication_IPCommunication.c"
#include "KeyboardHandler.c"
void initialize(void); //all the usual mcu stuff 
char InterprocessorCommunication_TimerInteruptHandler_Count;       
//**********************************************************
//timer 0 compare ISR
interrupt [TIM0_COMP] void timer0_compare(void)
{      
	InterprocessorCommunication_TimerInteruptHandler_Count++;
}  

//******************************* 


//**********************************************************       
//Entry point and task scheduler loop
void main(void)
{ 
	char count = 2;
	char Data[3];
	initialize();

	//main task scheduler loop  
	while(1) // Finite State Machine
	{   
		/*
			Since we couldn't use the operating system, we chose instead to use
			timer interupts and a main infinite loop.  This works, but is not as clean
			and un-obtrusive.

			Also, some other application code is mixed in, in that we test/deboubnce
			the keypad in this "timer cycle".
		*/
		if(InterprocessorCommunication_TimerInteruptHandler_Count > 30)
		{                                            
			InterprocessorCommunication_TimerInteruptHandler();      	
			InterProcessorCommunications_IPLayerThread_FromLeft();
			InterProcessorCommunications_IPLayerThread_FromRight(); 

			// insert code here to send a byte if you press the keypad ... 
			KeyboardHandler_Test();

			InterprocessorCommunication_TimerInteruptHandler_Count = 0; 
		}
		
		/*
			The appplication begins here. (except for the keypad handler)
		*/
		// did the user enter a number to send to another processor, if so, send it
		if(FilledKeypadBuffer)
		{
			Data[0] = CurrentStringBuffer[1];          
			Data[1] = 0;
			InterProcessorCommunication_SendPacket(Data, CurrentStringBuffer[0]);
			FilledKeypadBuffer = 0;
		}
		// have to check if there is data, as we don't have the semaphore constructs.
		// did we receive data, if so, we need to display it on the LEDs.
		while(InterProcessorCommunicationPacketQueue_HasRead())
		{
			InterProcessorCommunicationPacketQueue_ReadData(Data);
			PORTC = Data[0];
		}
	}
}

//********************************************************** 
//Set it all up
void initialize(void)
{
	//set up the ports
	DDRA = 0b00110110;
	DDRD = 0b01100011;
	DDRC = 0xFF; // to the LEDs (will change to LCD);

	//set up timer 0  
	TIMSK=2;		//turn on timer 0 cmp match ISR 
	OCR0 = 250;  	//set the compare re to 250 time ticks
	//prescalar to 64 and turn on clear-on-match
	TCCR0=0b00001011;	                                         

	//init the task timers
	InterprocessorCommunication_TimerInteruptHandler_Count = 0; 

	// initialize the external interupt control bits
	//  See page 64 in the full documentation for more information
	// Also enable these interupts in teh GICR register (see docs on page 65-66)
	MCUCR = (MCUCR & 0xF0) | 0x0F; // sets the lower 4 bits to 1's
	GICR = (GICR & 0x3F) | 0xC0; // sets the higher bits to 1

	// initialize some of the Communications protocol stuff.
	InterProcessorCommunication_IPCommunication_Initialize();
	PORTA.2 = 1;
	PORTD.1 = 1; 

	// initialize the keypad stuff
	OldButtonState = 0;
	OldKeypadState = 0;
	FilledKeypadBuffer = 0;
	CurrentStringBuffer_CurrentIndex = 0;

	//crank up the ISRs
	#asm
		sei
	#endasm 
}  

